#include "lvgl.h"
#include "lv_port_disp.h"
#include "lv_port_indev.h"
#include "stdio.h"
#include "example.h"
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include "gui_guider.h"
#include "custom.h"
#include "widgets_init.h"
#include <modbus/modbus.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

lv_ui guider_ui;
modbus_t *ctx;

float t1, t2;
float h1, h2;

#ifdef NXDK
#define ROOT_PATH "A:D:"
#define TEST_FILE "A:D:\\text.txt"
#else
#define ROOT_PATH "A:."
#define TEST_FILE "A:text.txt"
#endif

#include <stdio.h>
void _log_cb(const char *buf)
{
    printf("%s", buf);
    fflush(stdout);
}

void modbus_init()
{
    // 创建并初始化 Modbus 连接
    ctx = modbus_new_rtu("/dev/ttyUSB0", 9600, 'N', 8, 1);
    if (ctx == NULL) {
        fprintf(stderr, "Unable to create the libmodbus context\n");
    }

    // 设置从站地址（例如，从站地址为 0x01)
    modbus_set_slave(ctx, 0x1);

    // 尝试打开串口
    if (modbus_connect(ctx) == -1) {
        fprintf(stderr, "Connection failed: %s\n", modbus_strerror(errno));
    }
}

// 定时器回调函数
void timer_handler(union sigval arg) {
    // 这里写定时器每次触发时执行的代码
    printf("定时器触发，执行任务...\n");

    {
        // 读取保持寄存器的数量和起始地址
        uint16_t reg[2]= {0, 0};  // 用于存储读取到的6个保持寄存器的值
        int addr = 0;     // 从寄存器地址 0 开始读取
        int num_regs = 2; // 读取 2 个寄存器
        // 设置从站地址（例如，从站地址为 0x01)
        modbus_set_slave(ctx, 0x1);
        modbus_read_registers(ctx, addr, num_regs, reg);

        t1 = reg[0] / 10.0f;
        h1 = reg[1] / 10.0f;
    }
    usleep(5000);
    {
        // 读取保持寄存器的数量和起始地址
        uint16_t reg[2] = {0, 0};  // 用于存储读取到的6个保持寄存器的值
        int addr = 0;     // 从寄存器地址 0 开始读取
        int num_regs = 2; // 读取 2 个寄存器
        // 设置从站地址（例如，从站地址为 0x02)
        modbus_set_slave(ctx, 0x2);
        modbus_read_registers(ctx, addr, num_regs, reg);

        t2 = reg[0] / 10.0f;
        h2 = reg[1] / 10.0f;
    }

    printf("T1: %02.1f H1: %02.1f\n", t1, h1);
    printf("T2: %02.1f H2: %02.1f\n", t2, h2);

    char buff[512];
    lv_ui *ui = &guider_ui;
    sprintf(buff, "从站 1: 温度: %02.1f℃ 湿度: %02.1f%%", t1, h1);
	lv_label_set_text(ui->screen_label_1, buff);
    sprintf(buff, "从站 2: 温度: %02.1f℃ 湿度: %02.1f%%", t2, h2);
	lv_label_set_text(ui->screen_label_2, buff);

    static int times = 0;
    if(times++ > 100)
        exit(0);
}

int main(int argc, char *argv[])
{
    lv_init();
    modbus_init();
    // lv_log_register_print_cb(_log_cb);
    lv_port_disp_init(800, 480);
    lv_port_indev_init(true);
    lv_fs_stdio_init();

    lv_fs_dir_t dir;
    lv_log("Listing files and folders with the executable folder\n");
    if (lv_fs_dir_open(&dir, ROOT_PATH) == LV_FS_RES_OK)
    {
        char fname[256];
        while (lv_fs_dir_read(&dir, fname) == LV_FS_RES_OK)
        {
            if (strlen(fname) == 0)
            {
                break;
            }
            lv_log("%s", fname);
            lv_log("\n");
        }
        lv_log("\n");
        lv_fs_dir_close(&dir);
    }
    else
    {
        lv_log("Couldnt open dir\n");
    }

    lv_fs_file_t file;
    do
    {
        const char *test_string = "hello world!\n";
        char read_back[256] = {0};
        uint32_t brw;
        if (lv_fs_open(&file, TEST_FILE, LV_FS_MODE_WR) != LV_FS_RES_OK)
        {
            lv_log("Could not open file for write\n");
            break;
        }

        if (lv_fs_write(&file, test_string, strlen(test_string), &brw) != LV_FS_RES_OK)
        {
            lv_log("Could not write to file\n");
            break;
        }

        lv_fs_close(&file);

        if (lv_fs_open(&file, TEST_FILE, LV_FS_MODE_RD) != LV_FS_RES_OK)
        {
            lv_log("Could not open file for read\n");
            break;
        }

        if (lv_fs_read(&file, read_back, strlen(test_string), &brw) != LV_FS_RES_OK)
        {
            lv_log("Could not read back file\n");
            break;
        }

        lv_log("Wrote: ");
        lv_log("%s", test_string);
        lv_log("\n");
        lv_log("Read back: ");
        lv_log("%s", read_back);
        lv_log("\n");
    } while (0);
    timer_t timer_id;
    struct sigevent sev;
    struct itimerspec ts;

    // 设置定时器的事件通知方式为信号通知，指定回调函数
    sev.sigev_notify = SIGEV_THREAD;      // 使用线程回调
    sev.sigev_notify_function = timer_handler; // 定时器到期时调用的回调函数
    sev.sigev_notify_attributes = NULL;   // 默认属性
    sev.sigev_value.sival_ptr = &timer_id; // 传递给回调函数的参数

    // 创建定时器
    if (timer_create(CLOCK_REALTIME, &sev, &timer_id) == -1) {
        perror("timer_create");
        exit(EXIT_FAILURE);
    }

    // 设置定时器：首次触发延迟 1 秒，周期为 1 秒
    ts.it_value.tv_sec = 1; // 初始延迟 1 秒
    ts.it_value.tv_nsec = 0;
    ts.it_interval.tv_sec = 1; // 周期 1 秒
    ts.it_interval.tv_nsec = 0;

    // 设置定时器
    if (timer_settime(timer_id, 0, &ts, NULL) == -1) {
        perror("timer_settime");
        exit(EXIT_FAILURE);
    }

    setup_ui(&guider_ui);
	custom_init(&guider_ui);

    while (lv_get_quit() == LV_QUIT_NONE)
    {
        lv_task_handler();
        // sleep(1);  // 主程序继续运行，定时器每 2 秒触发一次
    }

    lv_port_disp_deinit();
    lv_port_indev_deinit();
    lv_deinit();

}
